﻿// <copyright file="DiffObject.cs" company="Selectron Systems AG">
// Copyright (c) Selectron Systems AG. All rights reserved.
// </copyright>

namespace ObjectDiffer.Presentation.Evaluation.Model
{
    using System.Runtime.Serialization;

    using Telerik.Windows.Data;

    /// <summary>
    /// The difference mode
    /// </summary>
    public enum DiffType
    {
        /// <summary>
        /// No change
        /// </summary>
        None = 0,

        /// <summary>
        /// Changed entity
        /// </summary>
        Changed = 1,

        /// <summary>
        /// Inserted entity
        /// </summary>
        Inserted = 2,

        /// <summary>
        /// Deleted entity
        /// </summary>
        Deleted = 3
    }

    /// <summary>
    /// The Object tree containing differences.
    /// </summary>
    [DataContract]
    public class DiffObject
    {
        /// <summary>
        /// The children
        /// </summary>
        private RadObservableCollection<DiffObject> children;

        /// <summary>
        /// Initializes a new instance of the <see cref="DiffObject"/> class.
        /// </summary>
        public DiffObject()
        {
            this.children = new RadObservableCollection<DiffObject>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DiffObject"/> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public DiffObject(string left, string right)
            : this()
        {
            this.Left = left;
            this.Right = right;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DiffObject"/> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <param name="status">The status.</param>
        public DiffObject(string left, string right, DiffType status)
            : this(left, right)
        {
            this.Status = status;
        }

        /// <summary>
        /// Gets the children.
        /// </summary>
        /// <value>
        /// The children.
        /// </value>
        [DataMember]
        public RadObservableCollection<DiffObject> Children
        {
            get
            {
                if (this.children == null)
                {
                    this.children = new RadObservableCollection<DiffObject>();
                }

                return this.children;
            }
        }

        /// <summary>
        /// Gets or sets the left.
        /// </summary>
        /// <value>
        /// The left.
        /// </value>
        [DataMember]
        public string Left { get; set; }

        /// <summary>
        /// Gets or sets the right.
        /// </summary>
        /// <value>
        /// The right.
        /// </value>
        [DataMember]
        public string Right { get; set; }

        /// <summary>
        /// Gets or sets the status.
        /// </summary>
        /// <value>
        /// The status.
        /// </value>
        [DataMember]
        public DiffType Status { get; set; }

        /// <summary>
        /// Gets a value indicating whether [is expanded].
        /// </summary>
        /// <value>
        ///   <c>true</c> if [is expanded]; otherwise, <c>false</c>.
        /// </value>
        public bool IsExpanded
        {
            get
            {
                return this.Status != DiffType.None;
            }
        }
    }
}
